Learn R Programming

r2sundials (version 7.2.1-4)

get_cnst: Get Predefined Constant

Description

This function returns numerical value of predefined named constants. Constants defined for this package are:

  • CV_ADAMS

  • CV_BDF

  • CV_SIMULTANEOUS

  • CV_STAGGERED

  • CV_STAGGERED1

  • CV_SUCCESS

  • R2SUNDIALS_EVENT_HOLD

  • R2SUNDIALS_EVENT_IGNORE

  • R2SUNDIALS_EVENT_STOP

All these constants are exported from the package and available for use in callback functions written in R.

Usage

get_cnst(s)

Value

integer scalar, the value of the constant named by s

Arguments

s

character scalar, name of the constant whose value is to get.

Examples

Run this code
get_cnst("CV_SUCCESS")
# [1] 0

Run the code above in your browser using DataLab